<?php

/*

dia2postgresql.pl php library : d2p_data class

This class reprents a table row.

Copyright (C) 2002 E.BENOIT   <tseeker@nocternity.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/


class	d2p_data
{
	var	$objId;
	var	$manager;
	var	$table;

	var	$fields;
	var	$modified;
	var	$state;
	var	$errorMsg;
 	var	$isLoaded;

	// Initializes a new data object ; the table argument is a d2p_table
	// class instance. The identifier argument may be NULL for a new object,
	// or a value or array for an existing database row.
	function	d2p_data(&$table, $identifier = NULL)
	{
		$this->objId = uniqid("d2p_data-", true);
		$this->table = &$table;
		$this->manager = &$this->table->manager;
		$this->modified = false;
 		$this->isLoaded = false;
		$this->errorMsg = "";

		if	(is_array($identifier))
		{
			if	(!$this->loadDatabaseEntry($identifier))
				$this->state = "i";
		}
		else if	(!is_null($identifier) && count($this->table->identifier) == 1)
		{
			$tmp = array();
			$tmp[$this->table->identifier[0]] = $identifier;
			if	(!$this->loadDatabaseEntry($tmp))
				$this->state = "i";
		}
		else if	(is_null($identifier))
		{
			$this->prepareFields();
			$this->state = "n";
		}
		else
		{
			$this->errorMsg = "Invalid identifier";
			$this->state = "i";
		}
	}

	// Gets the contents of a field. If the requested field references another
	// object, a d2p_data object is returned.
	function	&getField($fieldName)
	{
		if	(	$this->state !== "i" && $this->state !== "s"
			&&	is_array($this->table->fields[$fieldName])
			)
		{
			if	(	$this->table->getFieldReference($fieldName)
				&&	!is_null($this->fields[$fieldName]["current_value"])
				)
			{
				return	$this->manager->objectFromBase(
						$this->table->getFieldReference($fieldName),
						$this->fields[$fieldName]["current_value"]
					);
			}
			else
				return	$this->fields[$fieldName]["current_value"];
		}
		return	NULL;
	}

	// Gets the exact value of a field.
	function	getFieldValue($fieldName)
	{
		if	(	$this->state !== "i" && $this->state !== "s"
			&&	is_array($this->table->fields[$fieldName])
			)
			return	$this->fields[$fieldName]["current_value"];
		return	NULL;
	}

	// Sets the value of a field ; if the field references another table, the value may be either
	// a scalar (if you want to set it manually) or a reference to another d2p_data object.
	function	setField($fieldName, $value)
	{
		if	(	is_array($this->table->fields[$fieldName])
			&&	$this->state != "i" && $this->state != "s"
			)
		{
			if	(is_null($value) && $this->table->isFieldMandatory($fieldName))
				return	false;

			if	(	$this->table->getFieldReference($fieldName)
				&&	is_object($value)
				&&	!strcmp($value->table->name, $this->table->name)
				)
				$this->fields[$fieldName]["current_value"] = $value->getIdentifier();
			elseif	(!is_null($value))
				$this->fields[$fieldName]["current_value"] = preg_replace("/\\\\'/", "'", $value);
			else
				$this->fields[$fieldName]["current_value"] = NULL;

			$this->modified = true;
			$this->table->objectModified(&$this);
			return	true;
		}
		return	false;
	}

	// Marks the object (and its attached database row) for deletion
	function	destroy()
	{
		if	($this->state != "m")
			return	false;
		$this->state = "s";
		$this->modified = true;
		$this->table->objectModified(&$this);
	}

	// Returns the value of the row's identifier
	function	getIdentifier()
	{
		$tmp = array();
		foreach ($this->table->identifier as $idn)
			$tmp[$idn] = $this->getField($idn);
		return	$tmp;
	}

	// Generate the row's description.
	function	getDescription()
	{
		if	(!is_null($this->table->desc))
		{
			eval($this->table->desc);
			if	(!is_null($d) && $d != "")
				return	$d;
		}
		$tmp = $this->getIdentifier();
		$tmp2 = array();
		foreach ($tmp as $fn => $fv)
		{
			if	(is_object($tmp[$fn]))
				array_push($tmp2, $tmp[$fn]->getDescription());
			else
				array_push($tmp2, $fv);
		}
		return	join(", ", $tmp2);
	}

	// Returns a HTML form that will allow the current data to be edited
	function	getHTMLForm()
	{
		// FIXME : add XSLT-generated forms support
		$form = "";

		foreach	($this->fields as $fn => $f)
		{
			if	(!strcasecmp($this->getFieldType($fn), "serial"))
			{
				$form .= "<INPUT TYPE='hidden' NAME='form__";
				$form .= $this->table->getFieldTable($fn) . "__$fn' VALUE=\"";
				$form .= $this->getFieldValue($fn) . "\">\n";
			}
			elseif	(!strcasecmp($this->getFieldType($fn), "bool"))
			{
				$form .= "<INPUT TYPE='hidden' NAME='form_id__";
				$form .= $this->table->getFieldTable($fn) . "__$fn' VALUE=\"";
				$form .= $this->getFieldValue($fn) . "\">\n";

				$form .= "<TR><TD>\n";
				$form .= "<INPUT TYPE='checkbox' NAME='form__";
				$form .= $this->table->getFieldTable($fn) . "__$fn' VALUE=\"";
				if	(	$this->getField($fn) == 't'
					||	$this->getField($fn) == 'true'
					||	$this->getField($fn) == '1'
					)
					$form .= ' CHECKED';
				$form .= ">\n</TD><TD>\n";
				$tmpnam = preg_replace("/_/", " ", $fn);
				if	($this->table->isFieldMandatory($fn))
					$tmpnam = "<FONT COLOR='red'>$tmpnam</FONT>";
				$form .= "\t$tmpnam\n</TD></TR>\n";
			}
			else
			{
				$form .= "<INPUT TYPE='hidden' NAME='form_id__";
				$form .= $this->table->getFieldTable($fn) . "__$fn' VALUE=\"";
				$form .= $this->getFieldValue($fn) . "\">\n";

				$tmpnam = preg_replace("/_/", " ", $fn);
				$nosel = "";
				if	($this->table->isFieldMandatory($fn))
				{
					$tmpnam = "<FONT COLOR='red'>$tmpnam</FONT>";
					if	(is_null($this->getFieldValue($fn)))
						$nosel = "<OPTION VALUE=''>No record selected</OPTION>";
				}
				else
					$nosel = "<OPTION VALUE=''>No record selected</OPTION>";

				$form .= "<TR><TD>\n\t$tmpnam\n</TD><TD>\n";
				if	($this->table->getFieldReference($fn))
				{
					$tObj = &$this->manager->getTable($this->table->getFieldReference($fn));
					$dObj = &$this->getField($fn);
					$form .= $this->manager->getHTMLPopuplistFromBase(
						$tObj->name, "SELECT * FROM \"" . $tObj->pgName . "\"",
						$dObj, "form__" . $this->table->getFieldTable($fn) . "__$fn",
						"No records", $nosel
					);
				}
				elseif	(!strcasecmp($this->getFieldType($fn), "text"))
				{
					$form .= "<TEXTAREA WRAP='hard' COLS='80' ROWS='10' NAME='form__";
					$form .= $this->table->getFieldTable($fn) . "__$fn'>";
					$form .= $this->getField($fn) . "</TEXTAREA>";
				}
				else
				{
					$form .= "<INPUT TYPE='text' NAME='form__";
					$form .= $this->table->getFieldTable($fn) . "__$fn' VALUE=\"";
					$form .= $this->getField($fn);
					$form .= "\">\n";
				}
				$form .= "</TD></TR>\n";
			}
		}
		return	$form;
	}

	// Returns the type of a field
	function	getFieldType($fieldname)
	{
		return	$this->table->getFieldType($fieldname);
	}

	// Returns the name of the table a field has been defined in
	function	getFieldTable($fieldname)
	{
		return	$this->table->getFieldTable($fieldname);
	}

	// Returns true if the row has been modified.
	function	isModified()
	{
		return	$this->modified;
	}

	// Returns the current error message and reinitializes it.
	function	getError()
	{
		$err = $this->errorMsg;
		$this->errorMsg = "";
		return	$err;
	}

	// This method is used to validate a row after it's been modified.
	function	isValid()
	{
		$this->state = "m";
		$this->modified = false;
		foreach ($this->fields as $name => $details)
		{
			$this->fields[$name]["initial_value"] = $this->fields[$name]["current_value"];
		}
	}

	// Returns the object identifier
	function	getObjId()
	{
		return	$this->objId;
	}

	// Prepare the fields according to the table specification
	function	prepareFields()
	{
		$this->fields = array();
		foreach ($this->table->fields as $fn => $f)
		{
			$this->fields[$fn] = array(
				"initial_value"	=> NULL,
				"current_value" => NULL
			);
		}
	}

	// Loads a row from the database ; the identifier is a hash table containing the
	// value of each field that belongs to the identifier.
	function	loadDatabaseEntry($identifier)
	{
		$db = $this->manager->getDatabaseConnection();
		if	(is_null($db))
		{
			$this->errorMsg = "No database connection";
			return	false;
		}

		$query = "SELECT * FROM \"" . $this->table->pgName . "\" WHERE ";
		if	(!is_array($identifier))
		{
			$this->errorMsg = "Identifier should be an array";
			return	false;
		}

		$idn = $this->table->getIdentifier();
		$b = 0;
		foreach ($idn as $name)
		{
			if	($b)
				$query .= " AND ";
			else
				$b = 1;
			$query .= "\"$name\" = " . $this->fieldToSQL(
				$this->getFieldType($name), $identifier[$name]
			);
		}

		$result = @pg_query($db, $query);
		if      (!$result)
		{
			$this->errorMsg = "Unable to execute query : $query";
			return  false;
		}
		$row = pg_fetch_array($result);
		if	(!is_array($row))
		{
			$this->errorMsg = "Unable to fetch query result";
			return	false;
		}

		$this->state = "m";
		$this->fields = array();
		foreach ($this->table->fields as $fn => $f)
		{
			$this->fields[$fn] = array();
			$this->fields[$fn]["initial_value"] = $this->fields[$fn]["current_value"] = $row[$fn];
		}

 		return	($this->isLoaded = true);
	}

	// Gets the input from a getHTMLForm()-generated form ; may be used for new
	// objects as well as for already existing objects. Note that the second possibility
	// requires additionnal coding, you may want to use the manager's getObjectFromForm()
	// method instead.
	function	getFormInput()
	{
		foreach ($this->fields as $fieldName => $details)
		{
			if	(!is_array($details))
				continue;

			$ffName = "form__" . $this->getFieldTable($fieldName) . "__" . $fieldName;
			$ok = false; $val = "";
			$instr = "global \$$ffName; if (isset(\$$ffName)) { \$ok = true; \$val = \$$ffName; }";
			eval($instr);
			if	($ok)
			{
				if	(!strcasecmp($this->getFieldType($fieldName), "bool"))
					$val = '1';
				elseif	($val == "" && !$this->table->isFieldMandatory($fieldName))
					$val = NULL;
				$this->setField($fieldName, $val);
			}
			elseif	(!strcasecmp($this->getFieldType($fieldName), "bool"))
				$this->setField($fieldName, '0');
		}
	}

	// Generates a request that will flush the row's latest modifications into the database.
	// This method is being automatically called by the manager when necessary. You don't have
	// to call it from your code.
	function	createRequest()
	{
		$request = "";

		if	($this->state == "m" && $this->modified)
		{
			$request = "UPDATE \"" . $this->table->pgName . "\" SET ";
			$modif = "";
			foreach ($this->fields as $fieldName => $details)
			{
				if	(is_array($details) && $details["initial_value"] != $details["current_value"])
				{
					$request .= $modif;
					$request .= "\"$fieldName\" = ";
					$request .= $this->fieldToSQL(
						$this->getFieldType($fieldName), $details["current_value"]
					);
					$modif = ", ";
				}
			}
			if	($modif == "")
				$request = "";
			else
			{
				$request .= " WHERE ";
				$b = 0;
				foreach ($this->table->identifier as $name)
				{
					if	($b)
						$request .= " AND ";
					else
						$b = 1;
					$request .= "\"$name\" = " . $this->fieldToSQL(
						$this->getFieldType($name), $this->fields[$name]["initial_value"]
					);
				}
			}
		}
		elseif	($this->state == "n" && $this->modified)
		{
			$request .= "INSERT INTO \"" . $this->table->pgName . "\" (";
			$values = "";
			foreach ($this->fields as $fieldName => $details)
			{
				if	(is_array($details) && !is_null($details["current_value"]))
				{
					if	($values != "")
					{
						$request .= ", ";
						$values .= ", ";
					}
					if	(	strcasecmp($this->getFieldType($fieldName), "serial")
						||	$details["current_value"] > 0
						)
					{
						$request .= "\"$fieldName\"";
						$values .= $this->fieldToSQL(
							$this->getFieldType($fieldName), $details["current_value"]
						);
					}
				}
				elseif	(	is_array($details)
					&&	$this->table->isFieldMandatory($fieldName)
					&&	strcasecmp($this->getFieldType($fieldName), "serial")
					)
				{
					$this->errorMsg = "Missing mandatory fields";
					$request = "";
				}
			}
			if	($request != "")
			{
				if	($values == "")
				{
					$request = "";
					$this->errorMsg = "No value set for this record";
				}
				else
					$request .= ") VALUES ($values)";
			}
		}
		elseif	($this->state == "s" && $this->modified)
		{
			$request .= "DELETE FROM \"" . $this->table->pgName . "\" WHERE ";
			$b = 0;
			foreach ($this->table->identifier as $name)
			{
				if	($b)
					$request .= " AND ";
				else
					$b = 1;
				$request .= "\"$name\" = " . $this->fieldToSQL(
					$this->getFieldType($name), $this->fields[$name]["initial_value"]
				);
			}
		}

		return	$request;
	}

	// This method allows to create a SQL value from a field value
	// FIXME : should be easier to extend
	function	fieldToSQL($type, $value)
	{
		if	(is_null($value))
			return	"NULL";
		elseif	(preg_match("/.*int$/", $type))
			return	$value;
		elseif	(!strcasecmp($type, "serial"))
			return	($value > 0 ? $value : 0);
		elseif	($type == "bool")
			return	($value ? "true" : "false");
		else
			return	"'" . preg_replace("/\\'/", "''", $value) . "'";
	}
}


?>
