<?php

/*

dia2postgresql.pl php library : d2p_manager class

This class is the interface to dia2postgres' PHP library. It allows
you to query the base, create lists, etc... A d2p_manager instance
is created at the startup of a script using the library. It is 
available through the $d2pManager variable.

Copyright (C) 2002 E.BENOIT   <tseeker@nocternity.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

require_once "d2p_data.inc";
require_once "d2p_table.inc";

class	d2p_manager
{
	var	$dbConn;
	var	$lastModifier;
	var	$requests;
	var	$cache;
	var	$transaction;
	var	$tables;

	// Initialize a new Dia2Postgres object manager
	function	d2p_manager($connection)
	{
		$this->dbConn = $connection;
		$this->transaction = false;
		$this->lastModifier = NULL;
		$this->requests = array();
		$this->cache = array();
		$this->tables = array();
	}

	// Modify the current database connection
	function	setDatabaseConnection($connection)
	{
		if	($this->dbConn && count($this->requests) > 0)
			$this->rollback();
		$this->dbConn = $connection;
	}

	// Get the current database connection
	function	getDatabaseConnection()
	{
		return	$this->dbConn;
	}

	// Toggle transaction mode
	function	toggleTransactionMode()
	{
		if	($this->transaction && $this->dbConn)
			@pg_query($this->dbConn, "ROLLBACK TRANSACTION");
		elseif	(!$this->transaction && count($this->requests) > 0)
		{
			//FIXME
		}
		$this->transaction = !$this->transaction;
	}

	// Create a new table
	function	&newTable($super, $name, $pgName)
	{
		$tmp = &new d2p_table(&$this, $super, $name, $pgName);
		return	$tmp;
	}

	// Add a new table to the list ; you should not need to call this method, since it's automatically
	// done when you are using the newTable() method
	function	addTable(&$table)
	{
		if	(is_object($table) && is_a($table, "d2p_table") && !$this->tables[$table->name])
		{
			$this->tables[$table->name] = &$table;
			return	true;
		}
		return	false;
	}

	// Get a table description from the list
	function	&getTable($name)
	{
		return	$this->tables[$name];
	}

	// Sets the row description code for a table
	function	setTableDescription($table, $desc)
	{
		if	(!is_object($this->tables[$table]))
			return	false;
		$this->tables[$table]->setDescription($desc);
	}

	// Notify the manager that an object just got modified
	function	objectModified(&$object)
	{
		if	(!($object && is_object($object) && is_a($object, "d2p_data")))
			return	false;

		if	(	is_object($this->lastModifier)
			&&	($this->lastModifier->getObjId() != $object->getObjId())
			)
		{
			$req = $this->lastModifier->createRequest();
			if	($req != "")
			{
				if	($this->transaction && !@pg_query($this->dbConn, $req))
				{
					$this->rollback();
					return	false;
				}

				if	(preg_match("/^DELETE /", $req))
					unset($this->lastModifier);
				else
				{
					$this->lastModifier->isValid();
					if	(	$this->transaction
						&&	preg_match("/^INSERT /", $req)
						&&	!is_array($object->getIdentifierName())
						&&	!strcasecmp(
								$object->getFieldType($object->getIdentifierName()), "serial"
							)
						)
					{
						//FIXME
					}
				}
				array_push($this->requests, array("request" => $req, "object" => &$this->lastModifier));
			}
		}
		$this->lastModifier = &$object;

		return	true;
	}

	function	&newObject($type)
	{
		$tmp = NULL;
		if	(!is_object($this->tables[$type]))
			return	$tmp;
		return	$this->tables[$type]->newObject();
	}

	function	&objectFromBase($type, $identifier)
	{
		if	(!is_object($this->tables[$type]) || !$this->dbConn)
			return	NULL;
		return	$this->tables[$type]->objectFromBase($identifier);
	}

	function	&objectFromForm($type)
	{
		if	(!is_object($this->tables[$type]) || !$this->dbConn)
			return	NULL;

		$idn = $this->tables[$type]->getIdentifier();
		if	(count($idn) == 1)
		{
			if   (!strcasecmp(
				 $this->tables[$type]->getFieldType($idn[0]),
				 "serial"
			     ))
				$varname = 'form__';
			else
				$varname = 'form_id__';
			$varname .= $this->tables[$type]->getFieldTable($idn[0]);
			$varname .= "__" . $idn[0];

			eval(preg_replace(
				"/varname/", $varname,
				'global $varname; $tmp = $varname;'
			));
			$obj = &$this->tables[$type]->objectFromBase(array($idn[0] => $tmp));
		}
		else
		{
			$tmp = array();
			foreach ($idn as $n)
			{
				$cmd  = "global \$form_id__";
				$cmd .= $this->tables[$type]->getFieldTable($n);
				$cmd .= "__$n;";
				$cmd .= "\$tmp2 = \$form_id__";
				$cmd .= $this->tables[$type]->getFieldTable($n);
				$cmd .= "__$n;";
				eval($cmd);
				$tmp[$n] = $tmp2;
			}
			$obj = &$this->tables[$type]->objectFromBase($tmp);
		}
 		if	(is_object($obj) && $obj->isLoaded)
			$obj->getFormInput();
 		else
 			unset($obj);
		return	$obj;
	}

	function	&listFromBase($type, $request)
	{
		if	(!is_object($this->tables[$type]) || !$this->dbConn)
			return	array();
		$name = $this->tables[$type]->getIdentifier();
		if	(!is_array($name))
			return	array();

		$result = @pg_query($this->dbConn, $request);
		if	(!$result)
			return	array();

		$tab = array();
		while	($row = pg_fetch_array($result))
		{
			$id = array();
			foreach ($name as $pke)
			{
				$id[$pke] = $row[$pke];
			}
			$tmp = &$this->tables[$type]->objectFromBase($id);
			if	(!is_null($tmp))
				array_push($tab, &$tmp);
		}
		pg_free_result($result);

		return	$tab;
	}
	
	function	getHTMLTabFromBase($type, $request, $fields_showed)
	{
		$tab = &$this->listFromBase($type, $request);

		// Fields names are used as header for now
		$code = "  <TR>\n";
		foreach	($fields_showed as $fn)
			$code .= "    <TH>$fn</TH>\n";
		$code .= "  </TR>\n";
		
		if	(count($tab))
		{
			$t = &$tab[0]->table;
			if	(!is_object($t))
				return	$code;
		}

		for	($i=0;$i<count($tab);$i++)
		{
			$code .= " <TR>\n";
			foreach	($fields_showed as $fn)
			{
				if	($t->getFieldReference($fn))
				{
					$obj2 = &$tab[$i]->getField($fn);
					if	(!is_null($obj2))
						$tmp = $obj2->getDescription();
					else
						$tmp = "&nbsp;";
				}
				else
					$tmp = $tab[$i]->getField($fn);

				$code .= "  <TD>";
				$code .= ($tmp != "" ? $tmp : "&nbsp;");
				$code .= "</TD>\n";
			}
			$code .= " </TR>\n";
		}

		return	$code;
	}
	
	function	getHTMLPopuplistFromBase(
				$type, $request, &$selected, $selectName = "",
				$noRecordsText = "--", $options = "",
				$alwaysShowOptions = false, $elCode = '$t->getDescription()'
			)
	{
		if	($selectName == "")
			$selectName = $type;

		$tab = &$this->listFromBase($type, $request);
		$code = "<SELECT NAME='$selectName'>\n";

		if	(!count($tab))
		{
			$code .= "  <OPTION VALUE=''>$noRecordsText</OPTION>\n";
			if	($alwaysShowOptions)
				$code .= $options;
		}
		else
			$code .= $options;

		for	($i=0;$i<count($tab);$i++)
		{
			$idn = &$tab[$i]->getIdentifier();
			$tmp = array();

			for	($j=0;$j<count($idn);$j+=2)
			foreach	($idn as $f => $fv)
			{
				if	(is_object($idn[$f]))
				{
					$idx = &$idn[$f]->getIdentifier();
					array_push($tmp, $idx[0]);
				}
				else
					array_push($tmp, $idn[$f]);
			}

			$code .= '  <OPTION VALUE="' . join(",", $tmp) . '"';
			if	(is_object($selected) && $tab[$i]->getObjId() == $selected->getObjId())
				$code .= " SELECTED";
			$t = &$tab[$i];
			eval('$code .= ">" . (' . $elCode . ') . "</OPTION>\n";');
		}

	    	$code .= "</SELECT>\n";
		return	$code;
	}

	function	commit() //FIXME
	{
		if	(is_object($this->lastModifier))
		{
			$req = $this->lastModifier->createRequest();
			if	($req != "")
			{
				if	(preg_match("/^DELETE /", $req))
					unset($this->lastModifier);
				else
					$this->lastModifier->isValid();
				array_push($this->requests, array("request" => $req, "object" => &$this->lastModifier));
			}
		}

		if	(count($this->requests) == 0)
			return	true;

		if	($this->transaction)
			if	(!@pg_query($this->dbConn, "BEGIN TRANSACTION"))
				return	false;
		for	($i=0;$i<count($this->requests);$i++)
		{
			if	(!@pg_query($this->dbConn, $this->requests[$i]["request"]))
			{
				@pg_query($this->dbConn, "ROLLBACK TRANSACTION");
				return	false;
			}
		}
		if	(!@pg_query($this->dbConn, "COMMIT"))
			return	false;
		if	($this->transaction && !@pg_query($this->dbConn, "BEGIN TRANSACTION"))
			return	false;

		$this->requests = array();
		unset($this->lastModifier);
		return	true;
	}

	function	rollback()
	{
		//FIXME
	}
}

$d2pManager = new d2p_manager(NULL);


?>
