<?php

/*

dia2postgresql.pl php library : d2p_table class

This class reprents a table.

Copyright (C) 2002 E.BENOIT   <tseeker@nocternity.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

class	d2p_table
{
	var	$manager;
	var	$super;
	var	$name;
	var	$pgName;
	var	$fields;
	var	$identifier;
	var	$objects;
	var	$desc;

	// Initialize a new table representation
	function	d2p_table(&$manager, $super, $name, $pgName)
	{
		$this->manager = &$manager;
		$this->name = $name;
		$this->pgName = $pgName;

		if	($super)
			$this->super = &$this->manager->getTable($super);
		else
			$this->super = NULL;

		if	(is_object($this->super) && is_a($this->super, "d2p_table"))
		{
			$this->fields = $this->super->fields;
			$this->identifier = $this->super->identifier;
		}
		else
		{
			$this->fields = array();
			$this->identifier = NULL;
		}
		$this->objects = array();
		$this->manager->addTable(&$this);
		$this->desc = NULL;
	}

	// Add a field to the list
	function	addField($name, $type, $mandatory = false, $references = NULL)
	{
		if	(!is_array($this->fields[$name]))
			$this->fields[$name] = array(
				"type"		=> $type,
				"from_table"	=> $this->pgName,
				"mandatory"	=> $mandatory,
				"references"	=> $references
			);
	}

	// Set description for a field
	function	setFieldDescription($name, $description)
	{
		if	(is_array($this->fields[$name]))
			$this->fields[$name]["description"] = $description;
	}

	// Set the identifier field name
	function	setIdentifier($identifier)
	{
		if	($this->identifier)
			return	false;

		if	(is_array($identifier))
			$fieldlist = $identifier;
		elseif	(preg_match("/,/", $identifier))
			$fieldlist = explode(",", $identifier);
		else
			$fieldlist = array($identifier);

		foreach	($fieldlist as $fn)
		{
			if	(!is_array($this->fields[$fn]))
				return	false;
		}
		$this->identifier = $fieldlist;
		return	true;
	}

	// Notify the table that one of its rows just changed
	function	objectModified(&$object)
	{
		if	($object->table->name != $this->name)
			return	false;
		return	$this->manager->objectModified($object);
	}


	// Get the description for a field
	function	getFieldDescription($name)
	{
		if	(is_array($this->fields[$name]))
		{
			if	(!$this->fields[$name]["description"])
				$this->fields[$name]["description"] = "\$d = '$name';";
			return	$this->fields[$name]["description"];
		}
		return	"";
	}

	// Get the name of the table this field references
	function	getFieldReference($name)
	{
		if	(is_array($this->fields[$name]))
			return	$this->fields[$name]["references"];
		return	NULL;
	}

	// Get the table the field is really defined in
	function	getFieldTable($name)
	{
		if	(is_array($this->fields[$name]))
			return	$this->fields[$name]["from_table"];
		return	"";
	}

	// Get the type of a field
	function	getFieldType($name)
	{
		if	(is_array($this->fields[$name]))
			return	$this->fields[$name]["type"];
		return	"";
	}

	// Returns true if the field is mandatory
	function	isFieldMandatory($name)
	{
		if	(is_array($this->fields[$name]))
			return	$this->fields[$name]["mandatory"];
		return	false;
	}

	// Get the identifier field name
	function	getIdentifier()
	{
		return	$this->identifier;
	}

	// Get an object from the cache
	function	&getObject($identifier)
	{
		$idn = join(",", $identifier);
		return	$this->objects[$idn];
	}

	// Create a new empty object
	function	&newObject()
	{
		//$instr = '$tmp = new d2p_cl_' . $this->name . '($this, NULL);';
		//eval($instr);
		$tmp = &new d2p_data(&$this, NULL);
		if	(is_object($tmp) && is_a($tmp, "d2p_data"))
		{
			if	(count($this->identifier) == 1)
			{
				$idn = $this->identifier[0];
				$t = $this->fields[$idn]["type"];
				if	(!strcasecmp($t, "serial"))
				{
					for	($debcache=-1;is_object($this->objects[$debcache]);$debcache--)
						;
					$tmp->setField($idn, $debcache);
				}
			}
			return	$tmp;
		}
		unset($tmp);
		return	NULL;
	}

	// Create an object from a database record
	function	&objectFromBase($identifier)
	{
		if	(is_null($identifier))
			return	$this->newObject();
		else if	(!is_array($identifier))
			$idn = $identifier;
		else
			$idn = join(",", $identifier);
		if	(is_object($this->objects[$idn]))
			return	$this->objects[$idn];

		//$tmp = NULL;
		//$instr = '$tmp = new d2p_cl_' . $this->name . '($this, $identifier);';
		//eval($instr);
		$tmp = &new d2p_data(&$this, $identifier);
		if	(is_object($tmp) && is_a($tmp, "d2p_data") && $tmp->isLoaded)
			$this->objects[$idn] = &$tmp;
		else
			unset($tmp);
		return	$tmp;
	}

	// This method may be called to change the way a row's description is generated.
	// It takes a string containing PHP code that will be eval()'d ; this code must
	// set the $d variable to whatever it wants to, this $d variable will be returned
	// by a row's getDescription() method. In the code, $this references the row being
	// described. The description may be reset to the default by setting the argument
	// to NULL.
	function	setDescription($desc)
	{
		$this->desc = $desc;
	}
}


?>
