Dia2Postgres - PostgreSQL module documentation



The PostgreSQL module is able to generate SQL scripts that create the database
described by the diagram.


I. PostgreSQL command line arguments

-o <outfile>
--output <outfile>
	Outputs the PostgreSQL database creation script to the file <outfile>.
	If the file already exists, a backup copy is created as <outfile>~, and
	a portion of the code is read from the old file in order to be
	reinjected into the new one. See section II for more details.

-s <sqlfile>
--sqlfile <sqlfile>
	Adds the PostgreSQL script stored in file <sqlfile> to the generated
	script. You may add more than one file by repeating the argument, for
	example :

		dia2postgres -i foo.dia -s foo1.sql -s foo2.sql -s foo3.sql \
			-o foo_out.sql
	
	The scripts are added in the same order as on the command line. The
	previous line would have added foo1.sql, then foo2.sql then foo3.sql.


II. Generated script structure

The script generated by the PostgreSQL module has the following structure :

II.1. Header

	-- HEADER FROM DIAGRAM

	-- Input file  : <input file name>
	-- Output file : <outfile>

	-- Defines the following modules : 
	--  <List database modules defined by the diagram>

The following lines only appear if the modules defined by the diagram depend
on external database modules

	-- Requires the following modules :
	--  <List database modules the ones defined by the diagram depend on>

Then a new transaction is started :

	BEGIN TRANSACTION;


II.2. Generated script

If the diagram defines tables with enumerate fields, they are added into the
SQL script as tables :

	-- Enumeration table for table.field
	CREATE TABLE "e_table_field"(
		id	SERIAL PRIMARY KEY,
		txt	VARCHAR(6) NOT NULL
	);
	INSERT INTO "e_table_field"(txt) VALUES('VALUE');

The insertion above is repeated for each value the enumerate field may take.
The reference to the table thus created is then added to d2p_lst_list.

The real tables are then created, with their indice, for each database module :

	-- Tables for <module name>
	<tables creation code>


II.3. SQL files

If SQL files are to be injected into the output (via the -s command, see
Section I) :

	-- 
	-- ADDITIONAL SQL FILES
	-- 

And then for each file :

	-- 
	 -- SQL code from <sqlfile>
	-- 

	<code from sqlfile>


II.4. Additional SQL code

Then comes the comment that marks the beginning of the custom code section ;
this section is retrieved from a file being overwritten by the -o option
then pasted in the new one. If no file is being overwritten, an empty section
is created

	-- You may insert additionnal SQL code here (DO NOT REMOVE THIS LINE)

	<optional code>

	-- End of additionnal SQL code (DO NOT REMOVE THIS LINE)


II.5. Libraries update

Then the d2p SQL libraries data is updated :

	SELECT f_d2p_<library name>_update();


II.6. Footer

	COMMIT TRANSACTION;
	-- SQL code generated by dia2postgres.pl
	-- 
	-- on <date>


III. Using the generated scripts

Before trying to run any Dia2Postgres-generated database creation script,
you have to insert the PostgreSQL libraries that may be found in the
lib/ subdirectory of this package :

	\i lib/lst.sql
	\i lib/inh.sql
	\i lib/ref.sql

You may then use any generated database creation script :

	\i foo.sql
	\i bar.sql

(where foo.sql and bar.sql have been generated by Dia2Postgres)
