package D2P::Entity::Field::Enum;
use base 'D2P::Entity::Field';

use	D2P::Entity::Table;
use	D2P::Entity::Field;
use	D2P::Entity::Relation::Reference;

sub	init
{
	my	$this = shift;

	$this->SUPER::init(@_);

	my	$oft = $this->{fldType};
	$this->{fldType} = 'INT';

	$oft =~ s/^\s*\{\s*//;
	$oft =~ s/\s*\}\s*$//;
	$oft =~ s/\s//g;
	$this->{enumValues} = [ split(/,/, $oft) ];
	$this->{enumIndex} = {};
	for my $i (0 .. (@{$this->{enumValues}} - 1))
	{
		my	$val = $this->{enumValues}->[$i];
		$this->{enumIndex}->{$val} = $i;
	}
}


sub	default
{
	my	$this = shift;

	return $this->{enumIndex}->{$this->{default}} unless @_;

	my	$nv = shift;
	if	($nv eq "" || grep(/^$nv$/, @{$this->{enumValues}}))
	{
		$this->{default} = $nv;
		return	1;
	}
	return	0;
}


sub	preprocess
{
	my	$this	= shift;

	my	$tnam	= "e_";
	$tnam .= $this->{table}->name() . "_";
	$tnam .= $this->{fldName};

	my	$maxlen	= 0;
	foreach my $val (@{$this->{enumValues}})
	{
		$maxlen = length($val) if length($val) > $maxlen;
	}

	my	$etbl	= new D2P::Entity::Table($tnam);
	my	$efid	= new D2P::Entity::Field("id", "SERIAL");
	my	$eftx	= new D2P::Entity::Field("txt", "VARCHAR(" . ($maxlen + 1) . ")");
	$efid->notNull(1); $eftx->notNull(1);
	$etbl->diaId($tnam);
	$etbl->addField($efid, 1); $etbl->addField($eftx, 0);
	foreach my $ev (keys %{$this->{enumIndex}})
	{
		push @{$etbl->{contents}}, { txt => "'$ev'" };
	}

	my	$mod	= $this->{table}->module();
	$mod->addTable($etbl);
	$etbl->preprocess();

	my	$rel	= new D2P::Entity::Relation::Reference(
		$this->{table}->diaId(), $tnam,
		$this->{leftHdl}, $efid->leftHdl()
	);
	$rel->diaId("ref_$tnam");
	$rel->updateCascade(1);
	$rel->deleteCascade(1);
	$mod->addRelation($rel);
}


1;
