package D2P::Entity::Index;
use base qw(D2P::Entity);


sub	init
{
	my	$this = shift;

	$this->SUPER::init(@_);
	$this->{type} = 'index';
	$this->{isUnique} = 0;
	$this->{table} = undef;
	$this->{fields} = [];
	$this->{name} = "";
	$this->{leftHdl} = $this->{rightHdl} = -1;
	$this->{comment} = undef;
	$this->{unresolvedFields} = [];
}


sub	getFields
{
	my	$this = shift;
	return	@{$this->{fields}};
}


sub	addField
{
	my	$this = shift;
	my	$nfld = shift;

	return 0 if (!ref($nfld) || (ref($nfld) !~ /^D2P::Entity::Field(::.*)?$/));
	push @{$this->{fields}}, $nfld;

	return	1;
}


sub	addUnresolvedField
{
	my	$this	= shift;
	my	$nfldn	= shift;

	push @{$this->{unresolvedFields}}, $nfldn;
	return	1;
}


sub	findAllAncestors
{
	my	$this	= shift;
	my	@tables	= @_;
	my	@ntbl	= ();

	if	(@tables == 1)
	{
		@ntbl = map {$_->to()} @{$tables[0]->{ancestors}};
		@ntbl = ($tables[0], @ntbl, $this->findAllAncestors(@ntbl));
	}
	else
	{
		foreach my $t(@tables)
		{
			@ntbl = (@ntbl, $this->findAllAncestors($t));
		}
		@ntbl = (@tables, @ntbl);
	}

	return	@ntbl;
}


sub	postprocess
{
	my	$this = shift;

	$this->SUPER::postprocess();

	## List current table ancestors
	my	@allTables	= $this->findAllAncestors($this->table());

	## Find unresolved fields
	foreach my $fn (@{$this->{unresolvedFields}})
	{
		my	$fld = undef;

INNER:		foreach my $tbl (@allTables)
		{
			$fld = $tbl->fieldByName($fn);
			last INNER if (defined ($fld) && ref($fld) =~ /^D2P::Entity::Field(::.*)?$/);
		}

		if	(defined($fld))
		{
			push @{$this->{fields}}, $fld;
		}
		else
		{
			print STDERR "WARNING : field '$fn' not found in table '"
				. $this->table()->name() . "'\n";
		}
	}
}


1;
