package D2P::Entity::Module;
use base qw(D2P::Entity);


sub	init
{
	my	$this = shift;

	$this->SUPER::init(@_);
	$this->{type} = 'module';
	$this->{name} = shift;
	$this->{isExternal} = 0;
	$this->{schema} = undef;
	$this->{tables} = {
		byname	=> {},	# Table objects by name
		byid	=> {},	# Table objects by DIA id
		ordered	=> []	# Table objects sort by relations count
	};
	$this->{relations} = {};
	$this->{sequences} = {};
	$this->{comments} = {};
	$this->{commentsOrdered} = [];
	$this->{extRelCnt} = 0;
	$this->{modRelCnt} = 0;
}


sub	getTableByName
{
	my	($this, $tname) = @_;
	return undef unless defined $tname;
	return	$this->{tables}->{byname}->{$tname};
}


sub	getTableByDIAId
{
	my	($this, $tid) = @_;
	return undef unless defined $tid;
	return	$this->{tables}->{byid}->{$tid};
}


sub	addTable
{
	my	$this	= shift;
	my	$ntbl	= shift;

	return 0 if (!ref($ntbl) || (ref($ntbl) !~ /^D2P::Entity::Table(::.*)?$/));
	my	$tname	= $ntbl->name();
	return 0 if (!defined($tname) || $tname eq "");
	my	$tid	= $ntbl->diaId();
	return 0 if ($tid eq "");

	$this->{tables}->{byname}->{$tname} = $ntbl;
	$this->{tables}->{byid}->{$tid} = $ntbl;
	$ntbl->module($this);

	return	1;
}


sub	addRelation
{
	my	$this	= shift;
	my	$nrel	= shift;

	return 0 if (!ref($nrel) || (ref($nrel) !~ /^D2P::Entity::Relation(::.*)?$/));
	my	$rid	= $nrel->diaId();
	return 0 if ($rid eq "");

	$this->{relations}->{$rid} = $nrel;
	$nrel->module($this);

	return	1;
}


sub	addSequence
{
	my	$this	= shift;
	my	$nseq	= shift;

	return 0 if (!ref($nseq) || (ref($nseq) !~ /^D2P::Entity::Sequence(::.*)?$/));
	my	$sn	= $nseq->name();
	return 0 if ($sn eq "");

	$this->{sequences}->{$sn} = $nseq;
	$nseq->module($this);

	return	1;
}


sub	addComment
{
	my	$this	= shift;
	my	$ncom	= shift;

	return 0 if (!ref($ncom) || (ref($ncom) !~ /^D2P::Entity::Comment(::.*)?$/));
	my	$cid	= $ncom->diaId();
	return 0 if ($cid eq "");

	$this->{comments}->{$cid} = $ncom;
	push @{$this->{commentsOrdered}}, $ncom;
	$ncom->module($this);

	return	1;
}


sub	preprocess
{
	my	$this	= shift;

	die "ERROR : module has no name\n"
		unless $this->{name} ne "";

	foreach my $tid (keys %{$this->{tables}->{byid}})
	{
		my	$t = $this->{tables}->{byid}->{$tid};
		$t->preprocess();
	}
}


sub	postprocess
{
	my	$this	= shift;

	$this->SUPER::postprocess();
	foreach my $tid (keys %{$this->{tables}->{byid}})
	{
		my	$t = $this->{tables}->{byid}->{$tid};
		$t->postprocess();
	}
}


sub	reorder
{
	my	$this	= shift;
	my	(@all, @ordered);

	foreach my $tid (keys %{$this->{tables}->{byid}})
	{
		push @all, $this->{tables}->{byid}->{$tid};
	}

	@ordered = sort {
		($a->modRelCnt() == $b->modRelCnt())
			? ($a->extRelCnt() <=> $b->extRelCnt())
			: ($a->modRelCnt() <=> $b->modRelCnt())
	} @all;
	die "ERROR : tables have redundant dependencies inside module " . $this->name() . "\n"
		if $ordered[0]->modRelCnt();

	@{$this->{tables}->{ordered}} = @ordered;
}


1;
