package D2P::Entity::Relation::Reference;
use base qw(D2P::Entity::Relation);


sub	init
{
	my	$this = shift;

	$this->SUPER::init(@_);
	$this->{type} = 'Reference';
	$this->{cascadeUpdate} = 0;
	$this->{cascadeDelete} = 0;
	$this->{srcCnx} = @_[2];
	$this->{srcField} = undef;
	$this->{destCnx} = @_[3];
	$this->{destField} = undef;
	$this->{usePostgres} = 0;
}


sub	setTables
{
	my	($this, $source, $dest) = @_;

	$this->SUPER::setTables($source, $dest);

	foreach my $f1 ($source->getFields())
	{
		if	(	$f1->leftHdl() == $this->{srcCnx}
			||	$f1->rightHdl() == $this->{srcCnx}
			)
		{
			$this->{srcField} = $f1;
			last;
		}
	}
	die "ERROR : source field not found in reference\n"
		unless defined $this->{srcField};

	if	($this->{destCnx} < 8)
	{
		die "ERROR : referenced table " . $dest->name() . " has an invalid primary key\n"
			if	(@{$dest->{pkey}} != 1);
		$this->{destField} = $dest->{pkey}->[0];
	}
	else
	{
		foreach my $f2 ($dest->getFields())
		{
			if	(	$f2->leftHdl() == $this->{destCnx}
				||	$f2->rightHdl() == $this->{destCnx}
				)
			{
				$this->{destField} = $f2;
				last;
			}
		}
	}

	die "ERROR : destination field not found in reference from "
		. $this->srcField()->table()->name() . "." . $this->srcField()->fldName() . "\n"
			unless defined $this->{destField};

	my	$t1 = lc($this->srcField()->fldType());
	my	$t2 = lc($this->destField()->fldType());
	print STDERR "WARNING : type mismatch between " . $this->srcField()->table()->name() . "."
		. $this->srcField()->fldName() . " and " . $this->destField()->table()->name() . "."
		. $this->destField()->fldName() . "\n"
		unless ($t1 eq $t2 || ($t1 eq 'int' && $t2 eq 'serial'));

	die "ERROR : multiple references for field " . $this->srcField()->table()->name() . "."
		. $this->srcField()->fldName() . "\n"
			unless $this->srcField()->setReference($this->destField(), $this);

	#print "REFERENCE FROM "  . $this->srcField()->table()->name() . ".". $this->srcField()->fldName()
		#. " and " . $this->destField()->table()->name() . ".". $this->destField()->fldName() . "\n";
}


1;
