package D2P::Entity::Schema;
use base qw(D2P::Entity::Module);


sub	init
{
	my	$this = shift;

	$this->SUPER::init('Schema', @_);
	$this->{type} = 'Database schema';
	$this->{header} = "";
	$this->{modules} = {
		byname	=> {},
		byid	=> {},
		ordered	=> []
	};
	$this->{extModules} = {
		byname	=> {},
		byid	=> {}
	};
}


sub	addModule
{
	my	$this	= shift;
	my	$module	= shift;

	if	($module->name() eq "")
	{
		print STDERR "WARNING : name not set for module\n";
		return 0;
	}
	elsif	($module->diaId() eq "")
	{
		print STDERR "WARNING : DIA identifier not set for module\n";
		return 0;
	}
	elsif	(	defined($this->{modules}->{byname}->{$module->name()})
		||	defined($this->{extModules}->{byname}->{$module->name()})
		)
	{
		print STDERR "WARNING : a module named '" . $module->name() . "' already exists\n";
		return 0;
	}
	elsif	(	defined($this->{modules}->{byid}->{$module->diaId()})
		||	defined($this->{extModules}->{byid}->{$module->diaId()})
		)
	{
		print STDERR "WARNING : a module with id '" . $module->diaId() . "' already exists\n";
		return 0;
	}

	my	$dest = $module->isExternal() ? $this->{extModules} : $this->{modules};
	$dest->{byid}->{$module->diaId()} = $module;
	$dest->{byname}->{$module->name()} = $module;
	foreach my $tid (keys %{$module->{tables}->{byid}})
	{
		my	$t = $module->{tables}->{byid}->{$tid};
		$this->addTable($t);
		$t->module($module);
	}
	foreach my $rid (keys %{$module->{relations}})
	{
		my	$r = $module->{relations}->{$rid};
		$this->addRelation($r);
		$r->module($module);
	}
	foreach my $cid (keys %{$module->{comments}})
	{
		my	$c = $module->{comments}->{$cid};
		$this->{comments}->{$cid} = $c;
	}
	return 1;
}


sub	resolveRelations
{
	my	$this	= shift;
	my	%prio	= (
		"D2P::Entity::Relation::Comment"	=>	2,
		"D2P::Entity::Relation::Reference"	=>	1,
		"D2P::Entity::Relation::Inheritance"	=>	0
	);

	foreach my $relId (sort { $prio{ref($this->{relations}->{$a})} <=> $prio{ref($this->{relations}->{$b})} }
			keys %{$this->{relations}}
	)
	{
		my	$rel	= $this->{relations}->{$relId};

		if	($prio{ref($rel)} == 2)
		{
			# Handle comments
			my	$st	= $this->{tables}->{byid}->{$rel->fromId()};
			my	$dt	= $this->{comments}->{$rel->toId()};

			unless	(defined($st))
			{
				print STDERR "WARNING : Table " . $rel->fromId() . " not found for comment $relId\n";
				next;
			}
			unless	(defined($dt))
			{
				print STDERR "WARNING : Comment " . $rel->toId() . " not found for comment $relId\n";
				next;
			}

			$rel->setTables($st, $dt);
		}
		else
		{
			# Handle references and inheritance
			my	$st	= $this->{tables}->{byid}->{$rel->fromId()};
			my	$dt	= $this->{tables}->{byid}->{$rel->toId()};

			unless	(defined($st) && defined($dt))
			{
				print STDERR "WARNING : Tables not found for relation $relId\n";
				next;
			}

			if	(ref($rel->module()) =~ /::Schema$/)
			{
				$st->module()->addRelation($rel);
				if	($dt->module->isExternal())
				{
					$rel->module()->extRelCnt($rel->module()->extRelCnt() + 1);
				}
				else
				{
					$rel->module()->modRelCnt($rel->module()->modRelCnt() + 1);
				}
			}
			$rel->setTables($st, $dt);
		}
	}
}


sub	reorder
{
	my	$this	= shift;
	my	(@all, @ordered);

	foreach my $mid (keys %{$this->{modules}->{byid}})
	{
		push @all, $this->{modules}->{byid}->{$mid};
	}

	@ordered = sort {
		($a->modRelCnt() == $b->modRelCnt())
			? ($a->extRelCnt() <=> $b->extRelCnt())
			: ($a->modRelCnt() <=> $b->modRelCnt())
	} @all;
	die "ERROR : modules have redundant dependencies\n"
		if $ordered[0]->modRelCnt();

	@{$this->{modules}->{ordered}} = @ordered;

	foreach my $mod (@ordered)
	{
		$mod->reorder();
	}
}

sub	postprocess
{
	my	$this	= shift;

	## Resolve relations and sort modules/tables by creation order
	$this->resolveRelations();
	$this->reorder();

	## Postprocess modules
	foreach my $mod (@{$this->{modules}->{ordered}})
	{
		$mod->postprocess();
	}
}


1;
