package D2P::Entity::Table;
use base qw(D2P::Entity);


sub	init
{
	my	$this = shift;

	$this->SUPER::init(@_);
	$this->{type} = 'table';
	$this->{name} = shift;
	$this->{module} = undef;
	$this->{ancestors} = [];
	$this->{fields} = [];
	$this->{fieldsByName} = {};
	$this->{indices} = [];
	$this->{indicesByName} = {};
	$this->{pkey} = [];
	$this->{extRelCnt} = 0;
	$this->{modRelCnt} = 0;
	$this->{contents} = [];
	$this->{comment} = undef;
}


sub	getFields
{
	my	$this = shift;
	return	@{$this->{fields}};
}


sub	addField
{
	my	$this = shift;
	my	$nfld = shift;	# Field entity
	my	$pkey = shift;	# True indicates that the field is part of the primary key

	return 0 if (!ref($nfld) || (ref($nfld) !~ /^D2P::Entity::Field(::.*)?$/));
	push (@{$this->{fields}}, $nfld);
	$this->{fieldsByName}->{$nfld->fldName()} = $nfld;
	push (@{$this->{pkey}}, $nfld) if $pkey;
	$nfld->table($this);

	return	1;
}


sub	fieldByName
{
	my	$this = shift;
	my	$name = shift;
	return	defined($name) ? $this->{fieldsByName}->{$name} : undef;
}


sub	getIndices
{
	my	$this = shift;
	return	@{$this->{indices}};
}


sub	addIndex
{
	my	$this = shift;
	my	$nidx = shift;

	return 0 if (!ref($nidx) || (ref($nidx) !~ /^D2P::Entity::Index(::.*)?$/));
	push @{$this->{indices}}, $nidx;
	$this->{indicesByName}->{$nidx->name()} = $nidx;
	$nidx->table($this);

	return	1;
}


sub	indexByName
{
	my	$this = shift;
	my	$name = shift;
	return	defined($name) ? $this->{indicesByName}->{$name} : undef;
}


sub	addAncestor
{
	my	$this		= shift;
	my	$ancestor	= shift;

	return if grep { $_->diaId() eq $ancestor->diaId() } @{$this->{ancestors}};

	push @{$this->{ancestors}}, $ancestor;
	if	($ancestor->to()->module()->diaId() eq $this->module()->diaId())
	{
		$this->{modRelCnt} += 1;
	}
	else
	{
		$this->{extRelCnt} += 1;
	}
	return if @{$this->{pkey}};

	my	$table = $ancestor->to();
	return unless @{$table->{pkey}};
	@{$this->{pkey}} = @{$table->{pkey}};
}


sub	preprocess
{
	my	$this	= shift;
	my	($i, $pih);

	## Find handles for fields and indices
	for $i (0 .. (@{$this->{fields}} - 1))
	{
		my	$fld = $this->{fields}->[$i];
		$fld->leftHdl(8 + $i * 2);
		$fld->rightHdl($pih = (9 + $i * 2));
	}
	$pih += 1;
	for $i (0 .. (@{$this->{indices}} - 1))
	{
		my	$idx = $this->{indices}->[$i];
		$idx->leftHdl($pih + $i * 2);
		$idx->rightHdl($pih + 1 + $i * 2);
	}

	## Preprocess fields and indices
	for $i (0 .. (@{$this->{fields}} - 1))
	{
		my	$fld = $this->{fields}->[$i];
		$fld->preprocess();
	}
	foreach my $idx (@{$this->{indices}})
	{
		$idx->preprocess();
	}
}


sub	addRelation
{
	my	$this	= shift;
	my	$rel	= shift;

	if	(defined($rel->to()))
	{
		if	($rel->to()->module()->diaId() eq $this->module()->diaId())
		{
			$this->{modRelCnt} += 1;
		}
		else
		{
			$this->{extRelCnt} += 1;
		}
	}
}


sub	postprocess
{
	my	$this	= shift;

	$this->SUPER::postprocess();

	## Postprocess fields and indices
	for my $i (0 .. (@{$this->{fields}} - 1))
	{
		my	$fld = $this->{fields}->[$i];
		$fld->postprocess();
	}
	foreach my $idx (@{$this->{indices}})
	{
		$idx->postprocess();
	}
}


1;
