package	D2P::Generator::PgDrop;
use	strict;
use	base 'D2P::Generator';
use	D2P::Parser;
use	D2P::Utils;


sub	init
{
	my	$this	= shift;

	$this->SUPER::init();
	$this->{config}->{outfile} = "";
}


sub	commandLine
{
	my	$this	= shift;

	return	(
		"-d|--drop"	=> {
			desc	=> "Output a PostgreSQL script meant to destroy the database "
				. "to the specified file.",
			args	=> ["outfile"],
			uniq	=> 1
		}
	);
}


sub	generate
{
	my	$this	= shift;
	my	$parser	= shift;
	my	$schema	= $parser->{schema};

	my	$outfile = $this->{config}->{outfile};
	return unless $outfile;

	## Prepare output
	open(DROPOUT, ">$outfile");

	$this->utils(new D2P::Utils());
	$this->createSQLHeader($schema);
	$this->dropSQLTables($schema);
	$this->createSQLFooter($schema);
	$this->utils(undef);

	close DROPOUT;
}


###
## Create SQL header
#
sub	createSQLHeader
{
	my	$this	= shift;
	my	$sch	= shift;

	my	$title = $sch->header();
	$title =~ s/\n$//;
	my	@lines = split /\n/, $title;
	foreach my $l (@lines)
		{ print DROPOUT "-- $l\n"; }
	print DROPOUT "\n" if @lines;

	print DROPOUT "\n-- \n-- DROP SCRIPT\n-- \n\n";
	print DROPOUT "-- Will drop the following modules : \n";
	foreach my $mod (reverse @{$sch->modules()->{ordered}})
	{
		my	$title = $mod->name();
		$title =~ s/\s+/ /g;

		print DROPOUT "--  $title\n";
	}
	print DROPOUT "\n";
	print DROPOUT "BEGIN TRANSACTION;\n\n\n";
}


###
## Drop SQL tables for each included module
#
sub	dropSQLTables
{
	my	$this	= shift;
	my	$sch	= shift;

	foreach my $mod (reverse @{$sch->modules()->{ordered}})
	{
		my	$title = $mod->name();
		$title =~ s/\n/\n-- /g;

		## Drop tables
		print DROPOUT "-- Tables for $title\n\n";
		foreach my $t (reverse @{$mod->tables()->{ordered}})
		{
			my	$tnam = $this->utils()->strToPgId($t->name());
			(my	$tnVal = $tnam) =~ s/\"//g;

			print DROPOUT "DELETE FROM d2p_ref_list WHERE src_table = '$tnVal' OR dest_table = '$tnVal';\n";
			print DROPOUT "DELETE FROM d2p_inh_cache WHERE child = '$tnVal';\n";
			print DROPOUT "DELETE FROM d2p_lst_list WHERE d2p_name = '$tnVal';\n";
			print DROPOUT "DROP TABLE $tnam;\n";
			print DROPOUT "\n";
		}
		print DROPOUT "\n";

		## Drop sequences
		if	(keys %{$mod->sequences()})
		{
			my	@seqs = keys %{$mod->sequences()};

			print DROPOUT "-- Sequences for $title\n";
			foreach my $sn (@seqs)
			{
				my	$snam	= $this->utils()->strToPgId($sn);
				print DROPOUT "DROP SEQUENCE $snam;\n";
			}
			print DROPOUT "\n";
		}
	}
}


###
## Create SQL footer
#
sub	createSQLFooter
{
	my	$this	= shift;
	my	$sch	= shift;

	print DROPOUT "\n-- \n\n";
	print DROPOUT "SELECT f_d2p_inh_update();\nSELECT f_d2p_ref_update();\n\n";
	print DROPOUT "COMMIT TRANSACTION;\n\n";
	print DROPOUT "-- SQL drop script generated by dia2postgres.pl\n";
	print DROPOUT "-- $Main::VERSION\n"; 
	print DROPOUT "-- on " . `date`;
}



1;
